
#include <Si4735.h>
#include "Wire.h"
#include <Adafruit_SSD1306.h>
#include "TM3P.h"

Si4735 TM3P;
Adafruit_SSD1306 display(OLED_MOSI, OLED_CLK, OLED_DC, OLED_RESET, OLED_CS);
volatile byte State = 0;
boolean ManualTune = 1;


void setup()
{
  byte KEYS_PINS[KEYS_NUM] = {KEY_0_PIN, KEY_1_PIN, KEY_2_PIN, KEY_3_PIN,
                              KEY_4_PIN, KEY_5_PIN, KEY_6_PIN, KEY_7_PIN};

  for(byte i = 0; i < KEYS_NUM; i++)
  {
    pinMode(KEYS_PINS[i], INPUT);
    digitalWrite(KEYS_PINS[i], HIGH);
  }
  
  TM3P.begin();
  TM3P.setMode(FM);
  TM3P.setSpacing(10);
  TM3P.setVolume(57);
  TM3P.tuneFrequency(9520);
  while((STC_MASK & TM3P.getInterrupts()) == 0);
  
  display.begin(SSD1306_SWITCHCAPVCC);
  display.clearDisplay();
}
//-------------------------------------------------------------------------
void loop()
{
  ScanKeyBoard(); 
  
  if ((State & USER_FLAGS_MASK) != 0)
  {
    if ((State & VOLUME_UP_FLAG) == VOLUME_UP_FLAG)
    {
      TM3P.volumeUp();
      if (TM3P.getMute())
        TM3P.toggleMute();
      State &= ~VOLUME_UP_FLAG;
    }

    else if ((State & VOLUME_DOWN_FLAG) == VOLUME_DOWN_FLAG)
    {
      TM3P.volumeDown();
      if (TM3P.getMute())
        TM3P.toggleMute();
      State &= ~VOLUME_DOWN_FLAG;
    }

    else if ((State & FREQUENCY_UP_FLAG) == FREQUENCY_UP_FLAG)
    {
      if(!ManualTune)
        TM3P.seekUp();
      else
        TM3P.frequencyUp();
      while((STC_MASK & TM3P.getInterrupts()) == 0);
      TM3P.getFrequency(1);
      State &= ~FREQUENCY_UP_FLAG;
    }

    else if ((State & FREQUENCY_DOWN_FLAG) == FREQUENCY_DOWN_FLAG)
    {
      if(!ManualTune)
        TM3P.seekDown();
      else
        TM3P.frequencyDown();
      while((STC_MASK & TM3P.getInterrupts()) == 0);
      TM3P.getFrequency(1);
      State &= ~FREQUENCY_DOWN_FLAG;
    }

    else if ((State & MUTE_TOGGLE_FLAG) == MUTE_TOGGLE_FLAG)
    {
      TM3P.toggleMute();
      State &= ~MUTE_TOGGLE_FLAG;
    }

    else if ((State & MODE_CHANGE_FLAG) == MODE_CHANGE_FLAG)
    {
      byte NewMode = TM3P.getMode() + 1;
      if (NewMode > LW)
        NewMode = FM;

      TM3P.setMode(NewMode);
      if (NewMode == FM)
        TM3P.setSpacing(10);

      State &= ~MODE_CHANGE_FLAG;
    }

    else if ((State & MANUAL_TUNE_FLAG) == MANUAL_TUNE_FLAG)
    {
      if(ManualTune)
        ManualTune = false;
      else
        ManualTune = true;
      
      State &= ~MANUAL_TUNE_FLAG;
    }
  }

  UpdateDisplay();
  delay(250);
}
//---------------------------------------------------------------------
void UpdateDisplay()
{
  display.clearDisplay();
  display.setTextSize(STATUS_BAR_SIZE);
  display.setTextColor(WHITE);
  UpdateVolume();
  
  if(!ManualTune)
  {
    display.setCursor(TUNE_MODE_HOR_POS, TUNE_MODE_VER_POS);
    display.print("auto");
  }
  
  display.setCursor(FREQUENCY_HOR_POS, FREQUENCY_VER_POS);
  display.print(((float)TM3P.currentFrequency())/100);
  display.setCursor(MODE_HOR_POS, MODE_VER_POS);
  char ModeString[3] = "FM";
  byte TM3PMode = TM3P.getMode();
  switch(TM3PMode)
  {
    case AM:
      ModeString[0] = 'A';
    break;

    case SW:
      ModeString[0] = 'S';
      ModeString[1] = 'W';
    break;

    case LW:
      ModeString[0] = 'L';
      ModeString[1] = 'W';
    break;

    default:
    break;
  }
  
  display.print(ModeString);
  UpdateData();
  display.display();
}
//---------------------------------------------------------------------
void UpdateVolume()
{
  display.drawLine(VOLUME_HOR_POS, VOLUME_VER_POS+2, VOLUME_HOR_POS, VOLUME_VER_POS+4, WHITE);
  display.drawLine(VOLUME_HOR_POS, VOLUME_VER_POS+2, VOLUME_HOR_POS+4, VOLUME_VER_POS+2, WHITE);
  display.drawLine(VOLUME_HOR_POS, VOLUME_VER_POS+4, VOLUME_HOR_POS+4, VOLUME_VER_POS+4, WHITE);
  display.drawLine(VOLUME_HOR_POS+4, VOLUME_VER_POS+2, VOLUME_HOR_POS+4, VOLUME_VER_POS+4, WHITE);
  display.drawLine(VOLUME_HOR_POS+4, VOLUME_VER_POS+2, VOLUME_HOR_POS+9, VOLUME_VER_POS, WHITE);
  display.drawLine(VOLUME_HOR_POS+4, VOLUME_VER_POS+4, VOLUME_HOR_POS+9, VOLUME_VER_POS+6, WHITE);
  display.drawLine(VOLUME_HOR_POS+10, VOLUME_VER_POS, VOLUME_HOR_POS+10, VOLUME_VER_POS+6, WHITE);

  if (TM3P.getMute())
  {
    display.drawLine(VOLUME_VALUE_HOR_POS, VOLUME_VALUE_VER_POS+1, VOLUME_VALUE_HOR_POS+7, VOLUME_VALUE_VER_POS+5, WHITE);
    display.drawLine(VOLUME_VALUE_HOR_POS, VOLUME_VALUE_VER_POS+5, VOLUME_VALUE_HOR_POS+7, VOLUME_VALUE_VER_POS+1, WHITE);
  }
  else
  {
    display.setCursor(VOLUME_VALUE_HOR_POS, VOLUME_VALUE_VER_POS);
    display.print(TM3P.getVolume());
  }
}
//---------------------------------------------------------------------
void UpdateData()
{
  char UserDataString[] = "          Research and Development Center Chip&Dip          ";
  char DisplayString[DATA_CHAR_IN_LINE+1];
  char *StringPtr;

  static byte Start = 0;
  static byte End = DATA_CHAR_IN_LINE - 1;

  if (TM3P.checkRDS())
  {
    StringPtr = TM3P.rds.radioText;
  }
  else
  {
    StringPtr = UserDataString;
  }

  if(*(StringPtr + End) == '\0')
  {
    Start = 0;
    End = DATA_CHAR_IN_LINE - 1;
  }

  for(byte i = 0; i < DATA_CHAR_IN_LINE; i++)
  {
    DisplayString[i] = *(StringPtr + Start + i);
  }

  Start++;
  End++;
  
  display.setTextSize(DATA_SIZE);
  display.setCursor(DATA_HOR_POS, DATA_VER_POS);
  display.print(DisplayString);
}
//---------------------------------------------------------------------
//проверка клавиатуры
byte ScanKeyBoard()
{
  byte KEYS_PINS[KEYS_NUM] = {KEY_0_PIN, KEY_1_PIN, KEY_2_PIN, KEY_3_PIN,
                              KEY_4_PIN, KEY_5_PIN, KEY_6_PIN, KEY_7_PIN};

  for (byte i = 0; i < KEYS_NUM; i++)
  {
    if (digitalRead(KEYS_PINS[i]) == 0)
    {
      delay(70);
      if (digitalRead(KEYS_PINS[i]) == 0)
      {
        State |= 1 << i;
        return i;
      }
    }
  }
  
  return KEYBOARD_RELEASED;
}

